@@ -1,6 +1,7 @@
 import re
 import sys
 from functools import partial
+from distutils.version import LooseVersion
 
 from docutils import nodes, utils
 
@@ -33,6 +34,8 @@ def issue_nodelist(name, link=None):
     return signifier + hyperlink + [nodes.inline(text=":")] + trail
 
 
+release_line_re = re.compile(r'\((\d+\.\d+)\+\)') # e.g. '(1.2+)'
+
 def issues_role(name, rawtext, text, lineno, inliner, options={}, content=[]):
     """
     Use: :issue|bug|feature|support:`ticket_number`
@@ -61,9 +64,14 @@ def issues_role(name, rawtext, text, lineno, inliner, options={}, content=[]):
     # Additional 'new-style changelog' stuff
     if name in issue_types:
         nodelist = issue_nodelist(name, link)
+        line = None
         # Sanity check
         if ported not in ('backported', 'major', ''):
-            raise ValueError("Gave unknown issue metadata '%s' for issue no. %s" % (ported, issue_no))
+            match = release_line_re.match(ported)
+            if not match:
+                raise ValueError("Gave unknown issue metadata '%s' for issue no. %s" % (ported, issue_no))
+            else:
+                line = match.groups()[0]
         # Create temporary node w/ data & final nodes to publish
         node = issue(
             number=issue_no,
@@ -71,6 +79,7 @@ def issues_role(name, rawtext, text, lineno, inliner, options={}, content=[]):
             nodelist=nodelist,
             backported=(ported == 'backported'),
             major=(ported == 'major'),
+            line=line,
         )
         return [node], []
     # Return old style info for 'issue' for older changelog entries
@@ -126,22 +135,28 @@ class issue(nodes.Element):
 
     @property
     def backported(self):
-        return self['backported']
+        return self.get('backported', False)
 
     @property
     def major(self):
-        return self['major']
+        return self.get('major', False)
 
     @property
     def number(self):
         return self.get('number', None)
 
+    @property
+    def line(self):
+        return self.get('line', None)
+
     def __repr__(self):
         flag = ""
         if self.backported:
             flag = "backported"
         elif self.major:
             flag = "major"
+        elif self.line:
+            flag = self.line + "+"
         if flag:
             flag = " (%s)" % flag
         return "<%s #%s%s>" % (self.type, self.number, flag)
@@ -165,7 +180,7 @@ def construct_releases(entries, app):
     # Walk from back to front, consuming entries & copying them into
     # per-release buckets as releases are encountered. Store releases in order.
     releases = []
-    lines = {'unreleased_bugfix': [], 'unreleased_minor': []}
+    lines = {'unreleased_bugfix': [], 'unreleased_feature': []}
     # Also keep a master hash of issues by number to detect duplicates & assist
     # in explicitly defined release lists.
     issues = {}
@@ -204,10 +219,10 @@ def construct_releases(entries, app):
                 # removed from
                 for obj in entries:
                     if obj.type == 'bug':
-                        # Major bugfix: remove from unreleased_minor
+                        # Major bugfix: remove from unreleased_feature
                         if obj.major:
                             log("Removing #%s from unreleased" % obj.number)
-                            lines['unreleased_minor'].remove(obj)
+                            lines['unreleased_feature'].remove(obj)
                         # Regular bugfix: remove from bucket for this release's
                         # line + unreleased_bugfix
                         else:
@@ -217,12 +232,12 @@ def construct_releases(entries, app):
                             if obj in lines[line]:
                                 log("Removing #%s from %s" % (obj.number, line))
                                 lines[line].remove(obj)
-                    # Regular feature/support: remove from unreleased_minor
+                    # Regular feature/support: remove from unreleased_feature
                     # Backported feature/support: remove from bucket for this
-                    # release's line (if applicable) + unreleased_minor
+                    # release's line (if applicable) + unreleased_feature
                     else:
                         log("Removing #%s from unreleased" % obj.number)
-                        lines['unreleased_minor'].remove(obj)
+                        lines['unreleased_feature'].remove(obj)
                         if obj in lines.get(line, []):
                             lines[line].remove(obj)
 
@@ -235,17 +250,17 @@ def construct_releases(entries, app):
                     lines[line] = []
                     entries = [
                         x
-                        for x in lines['unreleased_minor']
+                        for x in lines['unreleased_feature']
                         if x.type in ('feature', 'support') or x.major
                     ]
                     releases.append({
                         'obj': focus,
                         'entries': entries
                     })
-                    lines['unreleased_minor'] = []
+                    lines['unreleased_feature'] = []
                 # Existing line -> empty out its bucket into new release.
                 # Skip 'major' bugs as those "belong" to the next release (and will
-                # also be in 'unreleased_minor' - so safe to nuke the entire
+                # also be in 'unreleased_feature' - so safe to nuke the entire
                 # line)
                 else:
                     log("pre-existing, making bugfix release")
@@ -257,17 +272,20 @@ def construct_releases(entries, app):
                     })
                     lines[line] = []
                     # Clean out the items we just released from
-                    # 'unreleased_bugfix'.  (Can't nuke it because there might be
-                    # some unreleased bugs for other release lines.)
+                    # 'unreleased_bugfix'.  (Can't nuke it because there might
+                    # be some unreleased bugs for other release lines.)
                     for x in entries:
                         if x in lines['unreleased_bugfix']:
                             lines['unreleased_bugfix'].remove(x)
         # Entries get copied into release line buckets as follows:
-        # * Features and support go into 'unreleased_minor' for use in new
+        # * Features and support go into 'unreleased_feature' for use in new
         # feature releases.
         # * Bugfixes go into all release lines (so they can be printed in >1
-        # bugfix release as appropriate) as well as 'unreleased_bugfix' (so they
-        # can be displayed prior to release')
+        # bugfix release as appropriate) as well as 'unreleased_bugfix' (so
+        # they can be displayed prior to release'). Caveats include bugs marked
+        # 'major' (they go into unreleased_feature instead) or with 'N.N+'
+        # (meaning they only go into release line buckets for that release and
+        # up.)
         # * Support/feature entries marked as 'backported' go into all
         # release lines as well, on the assumption that they were released to
         # all active branches.
@@ -282,8 +300,6 @@ def construct_releases(entries, app):
                 focus = issue(
                     type_='bug',
                     nodelist=issue_nodelist('bug'),
-                    backported=False,
-                    major=False,
                     description=[focus]
                 )
             else:
@@ -294,14 +310,21 @@ def construct_releases(entries, app):
             else:
                 issues[focus.number] = focus
             if focus.type == 'bug':
-                # Major bugs go into unreleased_minor
+                # Major bugs go into unreleased_feature
                 if focus.major:
-                    lines['unreleased_minor'].append(focus)
-                    log("Adding to unreleased_minor")
+                    lines['unreleased_feature'].append(focus)
+                    log("Adding to unreleased_feature")
                 # Regular bugs go into per-line buckets ('major' bugs do
-                # not as well as unreleased_bugfix
+                # not) as well as unreleased_bugfix. Adjust for bugs with a
+                # 'line' (minimum line no.) attribute.
                 else:
-                    for line in [x for x in lines if x != 'unreleased_minor']:
+                    bug_lines = [x for x in lines if x != 'unreleased_feature']
+                    if focus.line:
+                        bug_lines = [
+                            x for x in bug_lines
+                            if LooseVersion(x) >= LooseVersion(focus.line)
+                        ]
+                    for line in bug_lines:
                         log("Adding to %r" % line)
                         lines[line].append(focus)
             else:
@@ -314,15 +337,14 @@ def construct_releases(entries, app):
                 # Non-backported feature/support items go into feature releases
                 # only.
                 else:
-                    log("Adding to unreleased_minor")
-                    lines['unreleased_minor'].append(focus)
+                    log("Adding to unreleased_feature")
+                    lines['unreleased_feature'].append(focus)
 
     # Entries not yet released get special 'release' entries (that lack an
     # actual release object).
-    for which in ('bugfix', 'minor'):
-        name = 'feature' if which == 'minor' else which
+    for which in ('bugfix', 'feature'):
         nodelist = [release_nodes(
-            "Next %s release" % name,
+            "Next %s release" % which,
             'master',
             None,
             app.config
